@@ -8,10 +8,6 @@ import webbrowser
 from contextlib import suppress
 from pathlib import Path
 
-import click
-import toml
-from dotenv import cli as dotenv_cli
-
 from dynaconf import constants
 from dynaconf import default_settings
 from dynaconf import LazySettings
@@ -20,9 +16,12 @@ from dynaconf import settings as legacy_settings
 from dynaconf.loaders.py_loader import get_module
 from dynaconf.utils import upperfy
 from dynaconf.utils.files import read_file
+from dynaconf.utils.functional import empty
 from dynaconf.utils.parse_conf import parse_conf_data
 from dynaconf.validator import ValidationError
 from dynaconf.validator import Validator
+from dynaconf.vendor import click
+from dynaconf.vendor import toml
 
 
 CWD = Path.cwd()
@@ -44,7 +43,7 @@ def set_settings(ctx, instance=None):
         settings = import_settings(instance)
     elif "FLASK_APP" in os.environ:  # pragma: no cover
         with suppress(ImportError, click.UsageError):
-            from flask.cli import ScriptInfo
+            from flask.cli import ScriptInfo  # noqa
 
             flask_app = ScriptInfo().load_app()
             settings = flask_app.config
@@ -57,13 +56,13 @@ def set_settings(ctx, instance=None):
         sys.path.insert(0, os.path.abspath(os.getcwd()))
         try:
             # Django extension v2
-            from django.conf import settings
+            from django.conf import settings  # noqa
 
             settings.DYNACONF.configure()
         except (ImportError, AttributeError):
             # Backwards compatible with old django extension (pre 2.0.0)
             import dynaconf.contrib.django_dynaconf  # noqa
-            from django.conf import settings as django_settings
+            from django.conf import settings as django_settings  # noqa
 
             django_settings.configure()
             settings = django_settings
@@ -120,7 +119,9 @@ def split_vars(_vars):
     """Splits values like foo=bar=zaz in {'foo': 'bar=zaz'}"""
     return (
         {
-            upperfy(k.strip()): parse_conf_data(v.strip(), tomlfy=True)
+            upperfy(k.strip()): parse_conf_data(
+                v.strip(), tomlfy=True, box_settings=settings
+            )
             for k, _, v in [item.partition("=") for item in _vars]
         }
         if _vars
@@ -146,7 +147,7 @@ def print_version(ctx, param, value):
 def open_docs(ctx, param, value):  # pragma: no cover
     if not value or ctx.resilient_parsing:
         return
-    url = "http://dynaconf.readthedocs.io/"
+    url = "https://dynaconf.com/"
     webbrowser.open(url, new=2)
     click.echo(f"{url} opened in browser")
     ctx.exit()
@@ -197,7 +198,7 @@ def show_banner(ctx, param, value):
 @click.pass_context
 def main(ctx, instance):
     """Dynaconf - Command Line Interface\n
-    Documentation: http://dynaconf.readthedocs.io/
+    Documentation: https://dynaconf.com/
     """
     set_settings(ctx, instance)
 
@@ -370,7 +371,7 @@ def init(ctx, fileformat, path, env, _vars, _secrets, wg, y, django):
         )
 
     if django:  # pragma: no cover
-        dj_module, loaded_from = get_module({}, django)
+        dj_module, _ = get_module({}, django)
         dj_filename = dj_module.__file__
         if Path(dj_filename).exists():
             click.confirm(
@@ -470,12 +471,12 @@ def _list(env, key, more, loader, _all=False, output=None, flat=False):
     def color(_k):
         if _k in dir(default_settings):
             return "blue"
-        return "green"
+        return "magenta"
 
     def format_setting(_k, _v):
-        key = click.style(_k, bg=color(_k), fg="white")
+        key = click.style(_k, bg=color(_k), fg="bright_white")
         data_type = click.style(
-            f"<{type(_v).__name__}>", bg="bright_black", fg="white"
+            f"<{type(_v).__name__}>", bg="bright_black", fg="bright_white"
         )
         value = pprint.pformat(_v)
         return f"{key}{data_type} {value}"
@@ -491,10 +492,16 @@ def _list(env, key, more, loader, _all=False, output=None, flat=False):
             loaders.write(output, data, env=not flat and cur_env)
     else:
         key = upperfy(key)
-        value = data.get(key)
-        if not value:
+
+        try:
+            value = settings.get(key, empty)
+        except AttributeError:
+            value = empty
+
+        if value is empty:
             click.echo(click.style("Key not found", bg="red", fg="white"))
             return
+
         click.echo(format_setting(key, value))
         if output:
             loaders.write(output, {key: value}, env=not flat and cur_env)
