@@ -57,11 +57,20 @@ def load(django_settings_module_name=None, **kwargs):  # pragma: no cover
         k: v for k, v in django_settings_module.__dict__.items() if k.isupper()
     }
     options.update(kwargs)
-    options.setdefault("SKIP_FILES_FOR_DYNACONF", [settings_file])
+    options.setdefault(
+        "SKIP_FILES_FOR_DYNACONF", [settings_file, "dynaconf_merge"]
+    )
     options.setdefault("ROOT_PATH_FOR_DYNACONF", _root_path)
     options.setdefault("ENVVAR_PREFIX_FOR_DYNACONF", "DJANGO")
     options.setdefault("ENV_SWITCHER_FOR_DYNACONF", "DJANGO_ENV")
+    options.setdefault("ENVIRONMENTS_FOR_DYNACONF", True)
+    options.setdefault("load_dotenv", True)
+    options.setdefault(
+        "default_settings_paths", dynaconf.DEFAULT_SETTINGS_FILES
+    )
+
     lazy_settings = dynaconf.LazySettings(**options)
+    dynaconf.settings = lazy_settings  # rebind the settings
 
     # 2) Set all settings back to django_settings_module for 'django check'
     lazy_settings.populate_obj(django_settings_module)
@@ -78,22 +87,20 @@ def load(django_settings_module_name=None, **kwargs):  # pragma: no cover
             and (key != "SETTINGS_MODULE")
             and key not in lazy_settings.store
         ):
-            lazy_settings.logger.debug("Django default setting: %s", key)
             dj[key] = getattr(django_settings, key, None)
         dj["ORIGINAL_SETTINGS_MODULE"] = django_settings.SETTINGS_MODULE
 
     lazy_settings.update(dj)
 
     # 5) Patch django.conf.settings
-    class Wrapper(object):
+    class Wrapper:
 
         # lazy_settings = conf.settings.lazy_settings
 
         def __getattribute__(self, name):
             if name == "settings":
                 return lazy_settings
-            else:
-                return getattr(conf, name)
+            return getattr(conf, name)
 
     # This implementation is recommended by Guido Van Rossum
     # https://mail.python.org/pipermail/python-ideas/2012-May/014969.html
@@ -108,7 +115,6 @@ def load(django_settings_module_name=None, **kwargs):  # pragma: no cover
         ):
             stack_item.frame.f_globals["settings"] = lazy_settings
 
-    lazy_settings.logger.debug("Django Dynaconf: Finished loading")
     return lazy_settings
 
 
